@@ -29,6 +29,7 @@
 #include "macros.h"
 #include "enums.h"
 
+#include "tnl/tnl.h"
 #include "tnl/t_context.h"
 #include "intel_batchbuffer.h"
 
@@ -36,8 +37,10 @@
 #include "i915_context.h"
 #include "i915_program.h"
 
-#include "program_instruction.h"
+#include "prog_instruction.h"
+#include "prog_parameter.h"
 #include "program.h"
+#include "programopt.h"
 
 
 
@@ -59,7 +62,7 @@ static const GLfloat cos_constants[4] = {  1.0,
  */
 static GLuint src_vector( struct i915_fragment_program *p,
 			  const struct prog_src_register *source,
-			  const struct fragment_program *program )
+			  const struct gl_fragment_program *program )
 {
    GLuint src;
 
@@ -122,6 +125,7 @@ static GLuint src_vector( struct i915_fragment_program *p,
 	    p, p->ctx->FragmentProgram.Parameters[source->Index]);
 	 break;
 
+      case PROGRAM_CONSTANT:
       case PROGRAM_STATE_VAR:
       case PROGRAM_NAMED_PARAM:
          src = i915_emit_param4fv( 
@@ -244,7 +248,7 @@ do {									\
  */
 static void upload_program( struct i915_fragment_program *p )
 {
-   const struct fragment_program *program = p->ctx->FragmentProgram._Current;
+   const struct gl_fragment_program *program = p->ctx->FragmentProgram._Current;
    const struct prog_instruction *inst = program->Base.Instructions;
 
 /*    _mesa_debug_fp_inst(program->Base.NumInstructions, inst); */
@@ -779,9 +783,6 @@ static void fixup_depth_write( struct i915_fragment_program *p )
 }
 
 
-#define FRAG_BIT_TEX(n)  (FRAG_BIT_TEX0 << (n))
-
-
 static void check_wpos( struct i915_fragment_program *p )
 {
    GLuint inputs = p->FragProg.Base.InputsRead;
@@ -837,7 +838,7 @@ static void track_params( struct i915_fragment_program *p )
 
 static void i915BindProgram( GLcontext *ctx,
 			    GLenum target, 
-			    struct program *prog )
+			    struct gl_program *prog )
 {
    if (target == GL_FRAGMENT_PROGRAM_ARB) {
       i915ContextPtr i915 = I915_CONTEXT(ctx);
@@ -864,13 +865,13 @@ static void i915BindProgram( GLcontext *ctx,
    }
 }
 
-static struct program *i915NewProgram( GLcontext *ctx,
+static struct gl_program *i915NewProgram( GLcontext *ctx,
 				      GLenum target, 
 				      GLuint id )
 {
    switch (target) {
    case GL_VERTEX_PROGRAM_ARB:
-      return _mesa_init_vertex_program( ctx, CALLOC_STRUCT(vertex_program),
+      return _mesa_init_vertex_program( ctx, CALLOC_STRUCT(gl_vertex_program),
 					target, id );
 
    case GL_FRAGMENT_PROGRAM_ARB: {
@@ -893,7 +894,7 @@ static struct program *i915NewProgram( GLcontext *ctx,
 }
 
 static void i915DeleteProgram( GLcontext *ctx,
-			      struct program *prog )
+			      struct gl_program *prog )
 {
    if (prog->Target == GL_FRAGMENT_PROGRAM_ARB) {
       i915ContextPtr i915 = I915_CONTEXT(ctx);
@@ -909,7 +910,7 @@ static void i915DeleteProgram( GLcontext *ctx,
 
 static GLboolean i915IsProgramNative( GLcontext *ctx,
 				     GLenum target, 
-				     struct program *prog )
+				     struct gl_program *prog )
 {
    if (target == GL_FRAGMENT_PROGRAM_ARB) {
       struct i915_fragment_program *p = (struct i915_fragment_program *)prog;
@@ -925,7 +926,7 @@ static GLboolean i915IsProgramNative( GLcontext *ctx,
 
 static void i915ProgramStringNotify( GLcontext *ctx,
 				    GLenum target,
-				    struct program *prog )
+				    struct gl_program *prog )
 {
    if (target == GL_FRAGMENT_PROGRAM_ARB) {
       struct i915_fragment_program *p = (struct i915_fragment_program *)prog;
@@ -936,7 +937,15 @@ static void i915ProgramStringNotify( GLcontext *ctx,
        */
       ctx->Driver.Enable( ctx, GL_FRAGMENT_PROGRAM_ARB, 
 			  ctx->FragmentProgram.Enabled );
+
+      if (p->FragProg.FogOption) {
+         /* add extra instructions to do fog, then turn off FogOption field */
+         _mesa_append_fog_code(ctx, &p->FragProg);
+         p->FragProg.FogOption = GL_NONE;
+      }
    }
+
+   _tnl_program_string(ctx, target, prog);
 }
 
 
@@ -955,6 +964,17 @@ void i915ValidateFragmentProgram( i915ContextPtr i915 )
    GLuint s2 = S2_TEXCOORD_NONE;
    int i, offset = 0;
 
+   if (i915->current_program != p) 
+   {
+      if (i915->current_program) {
+	 i915->current_program->on_hardware = 0;
+	 i915->current_program->params_uptodate = 0;
+      }
+      
+      i915->current_program = p;
+   }
+
+
    /* Important:
     */
    VB->AttribPtr[VERT_ATTRIB_POS] = VB->NdcPtr;
@@ -996,7 +1016,10 @@ void i915ValidateFragmentProgram( i915ContextPtr i915 )
 	 EMIT_PAD( 1 );
    }
 
-#if 0
+   /* XXX this was disabled, but enabling this code helped fix the Glean
+    * tfragprog1 fog tests.
+    */
+#if 1
    if ((inputsRead & FRAG_BIT_FOGC) || i915->vertex_fog != I915_FOG_NONE) {
       EMIT_ATTR( _TNL_ATTRIB_FOG, EMIT_1F, S4_VFMT_FOG_PARAM, 4 );
    }
