@@ -49,7 +49,7 @@
 
 #include "swrast/swrast.h"
 #include "swrast_setup/swrast_setup.h"
-#include "array_cache/acache.h"
+#include "vbo/vbo.h"
 
 #include "tnl/tnl.h"
 #include "tnl/t_pipeline.h"
@@ -165,12 +165,6 @@ static const struct tnl_pipeline_stage *tdfx_pipeline[] = {
    &_tnl_texgen_stage, 
    &_tnl_texture_transform_stage, 
    &_tnl_point_attenuation_stage,
-#if 0
-#if defined(FEATURE_NV_vertex_program) || defined(FEATURE_ARB_vertex_program)
-   &_tnl_arb_vertex_program_stage,
-   &_tnl_vertex_program_stage,
-#endif
-#endif
    &_tnl_render_stage,		
    0,
 };
@@ -328,7 +322,7 @@ GLboolean tdfxCreateContext( const __GLcontextModes *mesaVis,
    /* Initialize the software rasterizer and helper modules.
     */
    _swrast_CreateContext( ctx );
-   _ac_CreateContext( ctx );
+   _vbo_CreateContext( ctx );
    _tnl_CreateContext( ctx );
    _swsetup_CreateContext( ctx );
 
@@ -609,7 +603,7 @@ tdfxDestroyContext( __DRIcontextPrivate *driContextPriv )
 
       _swsetup_DestroyContext( fxMesa->glCtx );
       _tnl_DestroyContext( fxMesa->glCtx );
-      _ac_DestroyContext( fxMesa->glCtx );
+      _vbo_DestroyContext( fxMesa->glCtx );
       _swrast_DestroyContext( fxMesa->glCtx );
 
       tdfxFreeVB( fxMesa->glCtx );
