@@ -45,7 +45,7 @@
 static void GLAPIENTRY _mesa_noop_EdgeFlag( GLboolean b )
 {
    GET_CURRENT_CONTEXT(ctx);
-   ctx->Current.EdgeFlag = b;
+   ctx->Current.Attrib[VERT_ATTRIB_EDGEFLAG][0] = (GLfloat)b;
 }
 
 static void GLAPIENTRY _mesa_noop_Indexf( GLfloat f )
