@@ -1,8 +1,8 @@
 /*
  * Mesa 3-D graphics library
- * Version:  6.5
+ * Version:  6.5.1
  *
- * Copyright (C) 1999-2005  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2006  Brian Paul   All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -37,6 +37,38 @@
 #include "texstore.h"
 
 
+#if FEATURE_EXT_texture_sRGB
+
+/**
+ * Convert an 8-bit sRGB value from non-linear space to a
+ * linear RGB value in [0, 1].
+ * Implemented with a 256-entry lookup table.
+ */
+static INLINE GLfloat
+nonlinear_to_linear(GLubyte cs8)
+{
+   static GLfloat table[256];
+   static GLboolean tableReady = GL_FALSE;
+   if (!tableReady) {
+      /* compute lookup table now */
+      GLuint i;
+      for (i = 0; i < 256; i++) {
+         const GLfloat cs = UBYTE_TO_FLOAT(i);
+         if (cs <= 0.04045) {
+            table[i] = cs / 12.92;
+         }
+         else {
+            table[i] = _mesa_pow((cs + 0.055) / 1.055, 2.4);
+         }
+      }
+      tableReady = GL_TRUE;
+   }
+   return table[cs8];
+}
+
+
+#endif /* FEATURE_EXT_texture_sRGB */
+
 
 /* Texel fetch routines for all supported formats
  */
@@ -79,6 +111,11 @@ static void fetch_null_texelf( const struct gl_texture_image *texImage,
 static void store_null_texel(struct gl_texture_image *texImage,
                              GLint i, GLint j, GLint k, const void *texel)
 {
+   (void) texImage;
+   (void) i;
+   (void) j;
+   (void) k;
+   (void) texel;
    /* no-op */
 }
 
@@ -245,54 +282,107 @@ const struct gl_texture_format _mesa_texformat_intensity = {
    store_texel_intensity		/* StoreTexel */
 };
 
-const struct gl_texture_format _mesa_texformat_depth_component_float32 = {
-   MESA_FORMAT_DEPTH_COMPONENT_FLOAT32,	/* MesaFormat */
-   GL_DEPTH_COMPONENT,			/* BaseFormat */
+
+#if FEATURE_EXT_texture_sRGB
+
+const struct gl_texture_format _mesa_texformat_srgb8 = {
+   MESA_FORMAT_SRGB8,			/* MesaFormat */
+   GL_RGB,				/* BaseFormat */
    GL_UNSIGNED_NORMALIZED_ARB,		/* DataType */
-   0,					/* RedBits */
-   0,					/* GreenBits */
-   0,					/* BlueBits */
+   8,					/* RedBits */
+   8,					/* GreenBits */
+   8,					/* BlueBits */
    0,					/* AlphaBits */
    0,					/* LuminanceBits */
    0,					/* IntensityBits */
    0,					/* IndexBits */
-   sizeof(GLfloat) * 8,			/* DepthBits */
-   sizeof(GLfloat) * 8,			/* StencilBits */
-   sizeof(GLfloat),			/* TexelBytes */
-   _mesa_texstore_depth_component_float32,/* StoreTexImageFunc */
+   0,					/* DepthBits */
+   0,					/* StencilBits */
+   3,					/* TexelBytes */
+   _mesa_texstore_srgb8,		/* StoreTexImageFunc */
    NULL,				/* FetchTexel1D */
-   NULL,				/* FetchTexel1D */
-   NULL,				/* FetchTexel1D */
-   fetch_texel_1d_f_depth_component_f32,/* FetchTexel1Df */
-   fetch_texel_2d_f_depth_component_f32,/* FetchTexel2Df */
-   fetch_texel_3d_f_depth_component_f32,/* FetchTexel3Df */
-   store_texel_depth_component_f32	/* StoreTexel */
+   NULL,				/* FetchTexel2D */
+   NULL,				/* FetchTexel3D */
+   fetch_texel_1d_srgb8,		/* FetchTexel1Df */
+   fetch_texel_2d_srgb8,		/* FetchTexel2Df */
+   fetch_texel_3d_srgb8,		/* FetchTexel3Df */
+   store_texel_srgb8			/* StoreTexel */
 };
 
-const struct gl_texture_format _mesa_texformat_depth_component16 = {
-   MESA_FORMAT_DEPTH_COMPONENT16,	/* MesaFormat */
-   GL_DEPTH_COMPONENT,			/* BaseFormat */
+const struct gl_texture_format _mesa_texformat_srgba8 = {
+   MESA_FORMAT_SRGBA8,			/* MesaFormat */
+   GL_RGBA,				/* BaseFormat */
+   GL_UNSIGNED_NORMALIZED_ARB,		/* DataType */
+   8,					/* RedBits */
+   8,					/* GreenBits */
+   8,					/* BlueBits */
+   8,					/* AlphaBits */
+   0,					/* LuminanceBits */
+   0,					/* IntensityBits */
+   0,					/* IndexBits */
+   0,					/* DepthBits */
+   0,					/* StencilBits */
+   4,					/* TexelBytes */
+   _mesa_texstore_srgba8,		/* StoreTexImageFunc */
+   NULL,				/* FetchTexel1D */
+   NULL,				/* FetchTexel2D */
+   NULL,				/* FetchTexel3D */
+   fetch_texel_1d_srgba8,		/* FetchTexel1Df */
+   fetch_texel_2d_srgba8,		/* FetchTexel2Df */
+   fetch_texel_3d_srgba8,		/* FetchTexel3Df */
+   store_texel_srgba8			/* StoreTexel */
+};
+
+const struct gl_texture_format _mesa_texformat_sl8 = {
+   MESA_FORMAT_SL8,			/* MesaFormat */
+   GL_LUMINANCE,			/* BaseFormat */
    GL_UNSIGNED_NORMALIZED_ARB,		/* DataType */
    0,					/* RedBits */
    0,					/* GreenBits */
    0,					/* BlueBits */
    0,					/* AlphaBits */
-   0,					/* LuminanceBits */
+   8,					/* LuminanceBits */
    0,					/* IntensityBits */
    0,					/* IndexBits */
-   sizeof(GLushort) * 8,		/* DepthBits */
-   sizeof(GLushort) * 8,		/* StencilBits */
-   sizeof(GLushort),			/* TexelBytes */
-   _mesa_texstore_depth_component16,	/* StoreTexImageFunc */
+   0,					/* DepthBits */
+   0,					/* StencilBits */
+   1,					/* TexelBytes */
+   _mesa_texstore_sl8,			/* StoreTexImageFunc */
    NULL,				/* FetchTexel1D */
-   NULL,				/* FetchTexel1D */
-   NULL,				/* FetchTexel1D */
-   fetch_texel_1d_f_depth_component16,	/* FetchTexel1Df */
-   fetch_texel_2d_f_depth_component16,	/* FetchTexel2Df */
-   fetch_texel_3d_f_depth_component16,	/* FetchTexel3Df */
-   store_texel_depth_component16	/* StoreTexel */
+   NULL,				/* FetchTexel2D */
+   NULL,				/* FetchTexel3D */
+   fetch_texel_1d_sl8,			/* FetchTexel1Df */
+   fetch_texel_2d_sl8,			/* FetchTexel2Df */
+   fetch_texel_3d_sl8,			/* FetchTexel3Df */
+   store_texel_sl8			/* StoreTexel */
 };
 
+const struct gl_texture_format _mesa_texformat_sla8 = {
+   MESA_FORMAT_SLA8,			/* MesaFormat */
+   GL_LUMINANCE_ALPHA,			/* BaseFormat */
+   GL_UNSIGNED_NORMALIZED_ARB,		/* DataType */
+   0,					/* RedBits */
+   0,					/* GreenBits */
+   0,					/* BlueBits */
+   8,					/* AlphaBits */
+   8,					/* LuminanceBits */
+   0,					/* IntensityBits */
+   0,					/* IndexBits */
+   0,					/* DepthBits */
+   0,					/* StencilBits */
+   2,					/* TexelBytes */
+   _mesa_texstore_sla8,			/* StoreTexImageFunc */
+   NULL,				/* FetchTexel1D */
+   NULL,				/* FetchTexel2D */
+   NULL,				/* FetchTexel3D */
+   fetch_texel_1d_sla8,			/* FetchTexel1Df */
+   fetch_texel_2d_sla8,			/* FetchTexel2Df */
+   fetch_texel_3d_sla8,			/* FetchTexel3Df */
+   store_texel_sla8			/* StoreTexel */
+};
+
+#endif /* FEATURE_EXT_texture_sRGB */
+
 const struct gl_texture_format _mesa_texformat_rgba_float32 = {
    MESA_FORMAT_RGBA_FLOAT32,		/* MesaFormat */
    GL_RGBA,				/* BaseFormat */
@@ -1117,6 +1207,54 @@ const struct gl_texture_format _mesa_texformat_z24_s8 = {
    store_texel_z24_s8			/* StoreTexel */
 };
 
+const struct gl_texture_format _mesa_texformat_z16 = {
+   MESA_FORMAT_Z16,			/* MesaFormat */
+   GL_DEPTH_COMPONENT,			/* BaseFormat */
+   GL_UNSIGNED_NORMALIZED_ARB,		/* DataType */
+   0,					/* RedBits */
+   0,					/* GreenBits */
+   0,					/* BlueBits */
+   0,					/* AlphaBits */
+   0,					/* LuminanceBits */
+   0,					/* IntensityBits */
+   0,					/* IndexBits */
+   sizeof(GLushort) * 8,		/* DepthBits */
+   0,					/* StencilBits */
+   sizeof(GLushort),			/* TexelBytes */
+   _mesa_texstore_z16,			/* StoreTexImageFunc */
+   NULL,				/* FetchTexel1D */
+   NULL,				/* FetchTexel1D */
+   NULL,				/* FetchTexel1D */
+   fetch_texel_1d_f_z16,		/* FetchTexel1Df */
+   fetch_texel_2d_f_z16,		/* FetchTexel2Df */
+   fetch_texel_3d_f_z16,		/* FetchTexel3Df */
+   store_texel_z16			/* StoreTexel */
+};
+
+const struct gl_texture_format _mesa_texformat_z32 = {
+   MESA_FORMAT_Z32,			/* MesaFormat */
+   GL_DEPTH_COMPONENT,			/* BaseFormat */
+   GL_UNSIGNED_NORMALIZED_ARB,		/* DataType */
+   0,					/* RedBits */
+   0,					/* GreenBits */
+   0,					/* BlueBits */
+   0,					/* AlphaBits */
+   0,					/* LuminanceBits */
+   0,					/* IntensityBits */
+   0,					/* IndexBits */
+   sizeof(GLuint) * 8,			/* DepthBits */
+   0,					/* StencilBits */
+   sizeof(GLuint),			/* TexelBytes */
+   _mesa_texstore_z32,			/* StoreTexImageFunc */
+   NULL,				/* FetchTexel1D */
+   NULL,				/* FetchTexel1D */
+   NULL,				/* FetchTexel1D */
+   fetch_texel_1d_f_z32,		/* FetchTexel1Df */
+   fetch_texel_2d_f_z32,		/* FetchTexel2Df */
+   fetch_texel_3d_f_z32,		/* FetchTexel3Df */
+   store_texel_z32			/* StoreTexel */
+};
+
 /*@}*/
 
 
@@ -1264,9 +1402,9 @@ _mesa_choose_tex_format( GLcontext *ctx, GLint internalFormat,
          case GL_DEPTH_COMPONENT:
          case GL_DEPTH_COMPONENT24_SGIX:
          case GL_DEPTH_COMPONENT32_SGIX:
-            return &_mesa_texformat_depth_component_float32;
+            return &_mesa_texformat_z32;
          case GL_DEPTH_COMPONENT16_SGIX:
-            return &_mesa_texformat_depth_component16;
+            return &_mesa_texformat_z16;
          default:
             ; /* fallthrough */
       }
@@ -1392,6 +1530,42 @@ _mesa_choose_tex_format( GLcontext *ctx, GLint internalFormat,
       }
    }
 
+#if FEATURE_EXT_texture_sRGB
+   if (ctx->Extensions.EXT_texture_sRGB) {
+      switch (internalFormat) {
+         case GL_SRGB_EXT:
+         case GL_SRGB8_EXT:
+            return &_mesa_texformat_srgb8;
+         case GL_SRGB_ALPHA_EXT:
+         case GL_SRGB8_ALPHA8_EXT:
+            return &_mesa_texformat_srgba8;
+         case GL_SLUMINANCE_EXT:
+         case GL_SLUMINANCE8_EXT:
+            return &_mesa_texformat_sl8;
+         case GL_SLUMINANCE_ALPHA_EXT:
+         case GL_SLUMINANCE8_ALPHA8_EXT:
+            return &_mesa_texformat_sla8;
+         /* NOTE: not supporting any compression of sRGB at this time */
+         case GL_COMPRESSED_SRGB_EXT:
+            return &_mesa_texformat_srgb8;
+         case GL_COMPRESSED_SRGB_ALPHA_EXT:
+            return &_mesa_texformat_srgba8;
+         case GL_COMPRESSED_SLUMINANCE_EXT:
+            return &_mesa_texformat_sl8;
+         case GL_COMPRESSED_SLUMINANCE_ALPHA_EXT:
+            return &_mesa_texformat_sla8;
+         case GL_COMPRESSED_SRGB_S3TC_DXT1_EXT:
+            return &_mesa_texformat_srgb8;
+         case GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT:
+         case GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT:
+         case GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT:
+            return &_mesa_texformat_srgba8;
+         default:
+            ; /* fallthrough */
+      }
+   }
+#endif /* FEATURE_EXT_texture_sRGB */
+
    _mesa_problem(ctx, "unexpected format in _mesa_choose_tex_format()");
    return NULL;
 }
