@@ -367,6 +367,20 @@ void x86_sub( struct x86_function *p,
    emit_op_modrm(p, 0x2b, 0x29, dst, src );
 }
 
+void x86_or( struct x86_function *p,
+             struct x86_reg dst,
+             struct x86_reg src )
+{
+   emit_op_modrm( p, 0x0b, 0x09, dst, src );
+}
+
+void x86_and( struct x86_function *p,
+              struct x86_reg dst,
+              struct x86_reg src )
+{
+   emit_op_modrm( p, 0x23, 0x21, dst, src );
+}
+
 
 
 /***********************************************************************
@@ -424,6 +438,14 @@ void sse_maxps( struct x86_function *p,
    emit_modrm( p, dst, src );
 }
 
+void sse_maxss( struct x86_function *p,
+		struct x86_reg dst,
+		struct x86_reg src )
+{
+   emit_3ub(p, 0xF3, X86_TWOB, 0x5F);
+   emit_modrm( p, dst, src );
+}
+
 void sse_divss( struct x86_function *p,
 		struct x86_reg dst,
 		struct x86_reg src )
@@ -456,6 +478,14 @@ void sse_mulps( struct x86_function *p,
    emit_modrm( p, dst, src );
 }
 
+void sse_mulss( struct x86_function *p,
+		struct x86_reg dst,
+		struct x86_reg src )
+{
+   emit_3ub(p, 0xF3, X86_TWOB, 0x59);
+   emit_modrm( p, dst, src );
+}
+
 void sse_addps( struct x86_function *p,
 		struct x86_reg dst,
 		struct x86_reg src )
@@ -629,6 +659,12 @@ void x87_fistp( struct x86_function *p, struct x86_reg dst )
    emit_modrm_noreg(p, 3, dst);
 }
 
+void x87_fild( struct x86_function *p, struct x86_reg arg )
+{
+   emit_1ub(p, 0xdf);
+   emit_modrm_noreg(p, 0, arg);
+}
+
 void x87_fldz( struct x86_function *p )
 {
    emit_2ub(p, 0xd9, 0xee);
@@ -1027,20 +1063,29 @@ struct x86_reg x86_fn_arg( struct x86_function *p,
 }
 
 
-void x86_init_func( struct x86_function *p )
-{
-   x86_init_func_size(p, 1024);
-}
-
-void x86_init_func_size( struct x86_function *p, GLuint code_size )
+/**
+ * Initialize an x86_function object, allocating space for up to
+ * 'code_size' bytes of code.
+ */
+GLboolean x86_init_func( struct x86_function *p, GLuint code_size )
 {
+   assert(!p->store);
    p->store = _mesa_exec_malloc(code_size);
-   p->csr = p->store;
+   if (p->store) {
+      p->csr = p->store;
+      return GL_TRUE;
+   }
+   else {
+      p->csr = NULL;
+      return GL_FALSE;
+   }
 }
 
 void x86_release_func( struct x86_function *p )
 {
-   _mesa_exec_free(p->store);
+   if (p->store)
+      _mesa_exec_free(p->store);
+   p->store = p->csr = NULL;
 }
 
 
